@@ -1,12 +1,12 @@
 =head1 NAME
 
-CGI::Application::Plugin::PageBuilder - Simplifies building pages with multiple templates.
+CGI::Application::Plugin::PageBuilder - Simplifies building pages with multiple templates in CGI::Application.
 
 =head1 SYNOPSIS
 
-This module is built on the idea that building complex web pages with the default CGI::Application method can get to be a real mess if you prefer to load many smaller templates to create your pages.
+This module is built on the idea that building complex web pages with the default CGI::Application method can get to be a real mess.  I personally found it much easier to build pages from many different smaller templates than to try to create one large template.  This is especially true when displaying large sets of data from a database where you would be filling in a lot of <TMPL_VAR>s for each cell.  This module aims to make that process a little easier.
 
-Now instead of
+So instead of:
 
  sub run_mode {
  	my $self = shift;
@@ -31,12 +31,17 @@ Now instead of
 
 You can do this:
 
- CGI:App subclass:
+ In your CGI:App subclass:
+
+ sub setup {
+     ...
+	 $self->pb_init( { header => 'header.tmpl', footer => 'footer.tmpl' } );
+	 ...
+ }
 
  sub run_mode {
   	my $self = shift;
 
-	$self->pb_template( 'header.tmpl' );
 	$self->pb_template( 'view_start.tmpl' );
 
  	my $db = MyApp::DB::Views->retrieve_all();
@@ -46,8 +51,6 @@ You can do this:
 		$self->pb_param( info, $line->info() );
  	}
 	$self->pb_template( 'view_end.tmpl' );
-	$self->pb_template( 'footer.tmpl' );
-
  	return $self->pb_build();
  }
 
@@ -59,13 +62,13 @@ Which arguably looks much cleaner.
 
 $self->pb_template( 'the_template_to_use.tmpl', ... );
 
-Adds the template to the page.  Any arguments past the template name are passed to HTML::Template. Returns the page contents.
+Adds the template to the page and sets it as the next template to apply param to.  Any agruments past the template name are passed to HTML::Template as arguments like "die_on_bad_params => 0", etc.
 
 =head2 pb_param
 
 $self->pb_param( name, value );
 
-Sets the value for the param in the template.  This applies to the last template loaded by B<pb_template()>.  Returns the page contents.
+Sets the value for the param in the template.  This applies to the last template loaded by B<pb_template()>.
 
 =head2 pb_build
 
@@ -91,7 +94,7 @@ This module is free software; you can redistribute it and/or modify it under the
 
 
 package CGI::Application::Plugin::PageBuilder;
-$VERSION = '0.9.1_1';
+$VERSION = '0.7';
 
 use Carp;
 
@@ -105,34 +108,70 @@ use vars '@EXPORT';
 			 pb_build
 			);
 
+use Data::Dumper;
+
+sub pb_init {
+	my $self = shift;
+	my $options = shift;
+
+	$self->{__PB_BUFFER} = '';
+	$self->{__PB_TEMPLATE_COUNT} = 0;
+	$self->{__PB_HEADER} = $options->{header} if exists( $options->{header} );
+	$self->{__PB_FOOTER} = $options->{footer} if exists( $options->{footer} );
+	$self->{__PB_READY} = 1;
+	$self->{__PB_TEMPLATE_LIST} = ();
+	$self->{__PB_TOOLBAR} = $options->{toolbar} if exists( $options->{toolbar} );
+	return $self;
+}
+
 sub pb_template {
 	my( $self, $template, %options ) = @_;
 
-	my $t_template = $self->load_tmpl( $template, %options );
-	return undef unless $t_template;
+	$self->pb_init() unless exists $self->{__PB_READY};
 
-	push( @{ $self->{__PB_TEMPLATE_LIST} }, $t_template );
 	$self->{__PB__TEMPLATE_COUNT}++;
-	return $self->pb_build();
+	my $status = $self->load_tmpl( $template, %options );
+	push( @{ $self->{__PB_TEMPLATE_LIST} }, $status );
+	return $status;
 }
 
 sub pb_build {
 	my $self = shift;
 
-	$self->{__PB_BUFFER} = '';
+	if ( exists $self->{__PB_HEADER} ) {
+		$self->pb_template( $self->{__PB_HEADER} );
+	}
+
+	if ( exists( $self->{__PB_TOOLBAR} ) ) {
+		$self->{__PB_BUFFER} .= $self->{__PB_TOOLBAR};
+	}
+
 	foreach my $template ( @{ $self->{__PB_TEMPLATE_LIST} } ) {
 		$self->{__PB_BUFFER} .= $template->output();
 	}
+
+	if ( exists $self->{__PB_FOOTER} ) {
+		$self->pb_template( $self->{__PB_FOOTER} );
+	}
 	return $self->{__PB_BUFFER};
 }
 
 sub pb_param {
-	my( $self, $param, $value ) = @_;
+	my( $self ) = shift;
+	my( $param ) = shift;
+
+	if ( ref( $param ) eq 'HASH' ) {
+		while ( my( $p, $v ) = each %{ $param } ) {
+			${${ $self->{__PB_TEMPLATE_LIST} }[$#{$self->{__PB_TEMPLATE_LIST}}]}->param( $p, $v );
+		}
+		return;
+	}
+
+	my $value = shift;
+	return undef unless( $value );
 
-	return undef unless $value;
 	${$self->{__PB_TEMPLATE_LIST}}[$#{@{$self->{__PB__TEMPLATE_LIST}}}]->param( $param, $value );
-	return $self->pb_build();
+	return 1;
 }
 
 1;
-
